#pragma once

#include <iostream>

#include <QtCore/QObject>
#include <QtWidgets/QLabel>

#include <QtNodes/NodeDelegateModelRegistry>
#include <QtNodes/NodeDelegateModel>

#include "PixmapData.hpp"

using QtNodes::PortType;
using QtNodes::PortIndex;
using QtNodes::NodeData;
using QtNodes::NodeDataType;
using QtNodes::NodeDelegateModel;

/// The model dictates the number of inputs and outputs for the Node.
/// In this example it has no logic.
class ImageLoaderModel : public NodeDelegateModel
{
  Q_OBJECT

public:
  ImageLoaderModel();

  ~ImageLoaderModel() = default;

public:

  QString
  caption() const override { return QString("Image Source"); }

  QString
  name() const override { return QString("ImageLoaderModel"); }

public:

  virtual QString
  modelName() const { return QString("Source Image"); }

  unsigned int
  nPorts(PortType const portType) const override;

  NodeDataType
  dataType(PortType const  portType,
           PortIndex const portIndex) const override;

  std::shared_ptr<NodeData>
  outData(PortIndex const port) override;

  void
  setInData(std::shared_ptr<NodeData>,
            PortIndex const portIndex) override {}

  QWidget*
  embeddedWidget() override { return _label; }

  bool
  resizable() const override { return true; }

protected:

  bool
  eventFilter(QObject* object, QEvent* event) override;

private:

  QLabel* _label;

  QPixmap _pixmap;
};
